// EditView.h
// (c) 2003-2004 exeal

#ifndef _EDIT_VIEW_H_
#define _EDIT_VIEW_H_

#include "EditDoc.h"
#include "..\..\Manah\DC.h"			// Manah::Windows::GDI::CMinimalDC
#include "..\..\Manah\MenuOperator.h"
#include "..\..\Manah\Text.h"
#include "..\..\Armaiti\DraggingTextDataObject.h"
#include "LineLayout.h"
#include "TextSearcher.h"
#include "KeyMacroPlayer.h"
#include "AutoCompleteWnd.h"
#include <set>
#include <algorithm>


namespace Ascension {

class CEditView;
class CEditController;
class CLineLayoutInfo;
class CLineLayoutManager;
class CEditPoint;
class CVisibleEditPoint;
class CKeywordManager;
class CKeyMacroPlayer;
class CAutoCompleteWnd;


// Miscellaneous
/////////////////////////////////////////////////////////////////////////////

///	ǐՒ Unicode ̃o[W
#define ASCENSION_UNICODE_VERSION	0x0400

///	ReLXgj[Ȃǂ ID
enum {
	WM_REDO	= WM_APP + 1,		///	ɖ߂
	WM_SELECTALL,				///	SđI
	ID_SHOWUNICODECONTROLCHARS,	///	Unicode 䕶̕\
	ID_SHOWRTL,					///	E獶ɓǂ
	ID_TOGGLEIMESTATUS,			///	IME J/
	ID_RECOMPOSITIONIME,		///	ĕϊ

	ID_INSERT_LRM,		///	LRM (Left-to-right mark)
	ID_INSERT_RLM,		///	RLM (Right-to-left mark)
	ID_INSERT_ZWJ,		///	ZWJ (Zero width joiner)
	ID_INSERT_ZWNJ,		///	ZWNJ (Zero width non-joiner)
	ID_INSERT_LRE,		///	LRE (Left-to-right embedding)
	ID_INSERT_RLE,		///	RLE (Right-to-left embedding)
	ID_INSERT_LRO,		///	LRO (Left-to-right override)
	ID_INSERT_RLO,		///	RLO (Right-to-left override)
	ID_INSERT_PDF,		///	PDF (Pop directional formatting)
	ID_INSERT_WJ,		///	WJ (Word Joiner)
	ID_INSERT_U2061,	///	U+2061 (Function Application)
	ID_INSERT_U2062,	///	U+2062 (Invisible Times)
	ID_INSERT_U2063,	///	U+2063 (Invisible Separator)
/*	ID_INSERT_NADS,		///	NADS (National digit shapes)	<- 񐄏vf炵
	ID_INSERT_NODS,		///	NODS (Nominal digit shapes)
	ID_INSERT_ASS,		///	ASS (Activate symmetric swapping)
	ID_INSERT_ISS,		///	ISS (Inhibit symmetric swapping)
	ID_INSERT_AAFS,		///	AAFS (Activate Arabic form shaping)
	ID_INSERT_IAFS,		///	IAFS (Inhibit Arabic form shaping)
*/	ID_INSERT_RS,		///	RS (Record Separator)
	ID_INSERT_US,		///	US (Unit Separator)

	ID_INSERT_U0020,	///	U+0020 (Space)
	ID_INSERT_NBSP,		///	NBSP (No-Break Space)
	ID_INSERT_U1680,	///	U+1680 (Ogham Space Mark)
	ID_INSERT_MVS,		///	MVS (Mongolian Vowel Separator)
	ID_INSERT_U2000,	///	U+2000 (En Quad)
	ID_INSERT_U2001,	///	U+2001 (Em Quad)
	ID_INSERT_U2002,	///	U+2002 (En Space)
	ID_INSERT_U2003,	///	U+2003 (Em Space)
	ID_INSERT_U2004,	///	U+2004 (Three-Per-Em Space)
	ID_INSERT_U2005,	///	U+2005 (Four-Per-Em Space)
	ID_INSERT_U2006,	///	U+2006 (Six-Per-Em Space)
	ID_INSERT_U2007,	///	U+2007 (Figure Space)
	ID_INSERT_U2008,	///	U+2008 (Punctuation Space)
	ID_INSERT_U2009,	///	U+2009 (Thin Space)
	ID_INSERT_U200A,	///	U+200A (Hair Space)
	ID_INSERT_ZWSP,		///	ZWSP (Zero Width Space)
	ID_INSERT_NNBSP,	///	NNSBP (Narrwow No-Break Space)
	ID_INSERT_MMSP,		///	MMSP (Medium Mathematical Space)
	ID_INSERT_U3000,	///	U+3000 (Ideographic Space)
	ID_INSERT_NEL,		///	NEL (Next Line)
	ID_INSERT_LS,		///	LS (Line Separator)
	ID_INSERT_PS,		///	PS (Paragraph Separator)
};

// ^C} ID
///	}EXhbOőI͈͂ύX
const UINT	ID_TIMER_EXPANDSELECTION		= 0;
///	}EXhbOőIsύX
const UINT	ID_TIMER_EXPANDLINESELECTION	= 1;
///	OLE hbO
const UINT	ID_TIMER_DRAGSCROLL				= 2;
///	ss
const UINT	ID_TIMER_LINEPARSE				= 3;

///	CWP[^}[W̕
#define INDICATORMARGIN_WIDTH	16

/**
 *	@brief	R}h̎
 *
 *	R}h̓GfB^̂鑀̒PʁBNCAg͂̃R}hŝ
 *	CEditView::ExecCommand gB܂AGfB^ł̓R}hPʂŃAhDA
 *	hDAL[}NL^/Đ/ۑsBeR}h̎ CEditView::CmdXXXX sB
 *	el̐̊ʓ̓R}ḧƂ̌^
 */
enum CommandIdentifier {
	///	I͈͂1O̕폜
	CMDID_EDIT_BACKSPACE,
	///	݂̉sR[hŉs
	CMDID_EDIT_BREAK,
	/// ̓ (̃R[h|Cg : unsigned long)
	CMDID_EDIT_CHAR,
	///	Rs[
	CMDID_EDIT_COPY,
	///	؂
	CMDID_EDIT_CUT,
	///	I͈͂1̕폜
	CMDID_EDIT_DELETE,
	///	ݍs폜
	CMDID_EDIT_DELETELINE,
	///	O̒P̐擪܂ō폜
	CMDID_EDIT_DELETETONEXTWORD,
	///	̒P̐擪܂ō폜
	CMDID_EDIT_DELETETOPREVWORD,
	///	1s}
	CMDID_EDIT_INSERTPREVLINE,
	///	I͈͂Ls^CY
	CMDID_EDIT_MAKESELCAPITAL,
	///	I͈͂ɕϊ
	CMDID_EDIT_MAKESELLOWER,
	///	I͈͂啶ɕϊ
	CMDID_EDIT_MAKESELUPPER,
	///	P͌
	CMDID_EDIT_OPENCANDIDATEWINDOW,
	///	\t
	CMDID_EDIT_PASTE,
	///	ĕϊ
	CMDID_EDIT_RECOMPOSESELECTION,
	///	蒼
	CMDID_EDIT_REDO,
	/// ͕̓ ###  (ϊ̎ : NextCharConvert)
	CMDID_EDIT_SETNEXTINPUTCONVERT,
	/// Xy[XCfg (Cfgx : unsigned short)
	CMDID_EDIT_SPACEINDENT,
	/// Xy[XtCfg (Cfgx : unsigned short)
	CMDID_EDIT_SPACEUNINDENT,
	///	I͈͂̋󔒂^uɕϊ
	CMDID_EDIT_TABIFY,
	/// ^uCfg (Cfgx : unsigned short)
	CMDID_EDIT_TABINDENT,
	/// ^utCfg (Cfgx : unsigned short)
	CMDID_EDIT_TABUNINDENT,
	///	̓ ( : wchar_t*)
	CMDID_EDIT_TEXT,
	///	 <-> R[h|Cg̕ϊ
	CMDID_EDIT_TOGGLECHARCODEPOINT,
	///	IME J/IME 
	CMDID_EDIT_TOGGLEIMESTATUS,
	///	}/㏑[h̕ύX
	CMDID_EDIT_TOGGLEOVERTYPEMODE,
	///	ɖ߂
	CMDID_EDIT_UNDO,
	///	I͈͂̃^u󔒂ɕϊ
	CMDID_EDIT_UNTABIFY,

	///	̃ubN}[N
	CMDID_MOVE_BOOKMARKNEXT,
	///	ÕubN}[N
	CMDID_MOVE_BOOKMARKPREV,
	///	I
	CMDID_MOVE_CANCELSELECTION,
	///	w蕶Ɉړ (ړ : unsigned long)
	CMDID_MOVE_CHARNEXT,
	///	w蕶܂őI (g : unsigned long)
	CMDID_MOVE_CHARNEXTEXTEND,
	///	w蕶OɈړ (ړ : unsigned long)
	CMDID_MOVE_CHARPREV,
	///	w蕶O܂őI (g : unsigned long)
	CMDID_MOVE_CHARPREVEXTEND,
	///	Ɉړ
	CMDID_MOVE_END,
	///	܂őI
	CMDID_MOVE_ENDEXTEND,
	///	Ɉړ
	CMDID_MOVE_HOME,
	///	܂őI
	CMDID_MOVE_HOMEEXTEND,
	/// wsɈړ (ړs : unsigned long)
	CMDID_MOVE_LINEDOWN,
	///	ws܂őI (gs : unsigned long)
	CMDID_MOVE_LINEDOWNEXTEND,
	///	sɈړ
	CMDID_MOVE_LINEEND,
	///	s܂őI
	CMDID_MOVE_LINEENDEXTEND,
	///	sɈړ
	CMDID_MOVE_LINEHOME,
	///	s܂őI
	CMDID_MOVE_LINEHOMEEXTEND,
	/// wsɈړ (ړs : unsigned long)
	CMDID_MOVE_LINEUP,
	///	ws܂őI (gs : unsigned long)
	CMDID_MOVE_LINEUPEXTEND,
	///	ΊʂɈړ
	CMDID_MOVE_MATCHBRACKET,
	///	Ίʂ܂ňړ
	CMDID_MOVE_MATCHBRACKETEXTEND,
	///	wy[WɈړ (ړy[W : unsigned long)
	CMDID_MOVE_PAGEDOWN,
	///	wy[W܂őI (gy[W : unsigned long)
	CMDID_MOVE_PAGEDOWNEXTEND,
	///	wy[WɈړ (ړy[W : unsigned long)
	CMDID_MOVE_PAGEUP,
	///	wy[W܂őI (gy[W : unsigned long)
	CMDID_MOVE_PAGEUPEXTEND,
	///	SđI
	CMDID_MOVE_SELECTALL,
	///	݂̒PI
	CMDID_MOVE_SELECTCURRENTWORD,
	///	̒P̏I[Ɉړ (ړꐔ : unsigned long)
	CMDID_MOVE_WORDENDNEXT,
	///	̒P̏I[܂őI (gꐔ : unsigned long)
	CMDID_MOVE_WORDENDNEXTEXTEND,
	///	O̒P̏I[Ɉړ (ړꐔ : unsigned long)
	CMDID_MOVE_WORDENDPREV,
	///	O̒P̏I[܂őI (gꐔ : unsigned long)
	CMDID_MOVE_WORDENDPREVEXTEND,
	///	̒P̐擪Ɉړ (ړꐔ : unsigned long)
	CMDID_MOVE_WORDNEXT,
	///	̒P̐擪܂őI (gꐔ : unsigned long)
	CMDID_MOVE_WORDNEXTEXTEND,
	///	O̒P̐擪Ɉړ (ړꐔ : unsigned long)
	CMDID_MOVE_WORDPREV,
	///	O̒P̐擪܂őI (gꐔ : unsigned long)
	CMDID_MOVE_WORDPREVEXTEND,

	///	1EփXN[
	CMDID_SCROLL_COLUMNNEXT,
	///	1񍶂փXN[
	CMDID_SCROLL_COLUMNPREV,
	///	ŏIsփXN[
	CMDID_SCROLL_END,
	///	Lbgɗ悤ɃXN[
	CMDID_SCROLL_ENSURECARETCENTERED,
	///	LbgɂȂ悤ɃXN[
	CMDID_SCROLL_ENSURECARETVISIBLE,
	///	擪sփXN[
	CMDID_SCROLL_HOME,
	///	1sփXN[
	CMDID_SCROLL_LINEDOWN,
	///	1sփXN[
	CMDID_SCROLL_LINEUP,
	///	1y[WփXN[
	CMDID_SCROLL_PAGEDOWN,
	///	1y[WփXN[
	CMDID_SCROLL_PAGEUP,
};

///	Cfg̎
enum AutoIndentType {
	///	Ȃ
	AIT_NONE,
	///	O̍sɈv
	AIT_BLOCK,
	///	`̃X}[gCfg
	AIT_SMART
};

/// ɓ͂镶̕ϊ
enum NextCharConvert {
	/// ϊ
	NCC_NONE,
	/// ANZg
	NCC_GRAVE,
	/// sANZg
	NCC_ACUTE,
	/// Ȑ܃ANZg
	NCC_CIRCUMFLEX,
	/// @
	NCC_TILDE,
	/// EEg
	NCC_DIAERESIS,
	/// ZfB[
	NCC_CEDILLA,
	///	t
	NCC_SUPERSCRIPT,
	///	t
	NCC_SUBSCRIPT
};

/// ҏW̎
enum EditOperationType {
	/// 
	EOT_NONE,
	/// ^CsO
	EOT_TYPING,
	/// 폜
	EOT_DELETE,
	/// ㏑
	EOT_REPLACE,
	/// \t
	EOT_PASTE,
};

///	CmdEditConvertSel ̕ϊ[h
enum SelConvertType {
	///	啶
	SCT_UPPERCASE,
	///	啶珬
	SCT_LOWERCASE,
	///	Ls^CY (P̐擪啶)
	SCT_CAPITALIZE,
};

/// ܂Ԃ[h
enum WrapMode {
	/// ܂ԂȂ
	WPM_NONE,
	/// w蕝Ő܂Ԃ
	WPM_SPECIFIED,
	/// EBhEŐ܂Ԃ
	WPM_WINDOW
};

/// [ɕ\ ()
enum LeftTabType {
	/// \Ȃ
	LTT_NONE		= 0x00,
	/// u[N|CgȂǂ\\ȃCWP[^}[W
	LTT_INDICATOR	= 0x01,
	/// sԍ
	LTT_LINENUMBER	= 0x02
};

/// C̎
enum UnderlineType {
	/// 
	ULT_NONE,
	/// 
	ULT_SOLID,
	/// j
	ULT_DASHED,
	/// _
	ULT_DOTTED,
	/// g
	ULT_WAVED
};

/// ҏWP
struct TEditOperation {
	EditOperationType	type;	// ̎
	CCharPos			pos;	// sʃLbgړʒu

	void	set(EditOperationType type_, const CCharPos& pos_) {
		type = type_;
		pos = pos_;
	}
};

/// ̐FAȂ
struct TTextFoundation {
	bool			enabled;	// C邩ǂ
	COLORREF		fgColor;	// OiF
	COLORREF		bgColor;	// wiF
	bool			italic;		// Α
	bool			bold;		// 
	UnderlineType	underline;	// 

	TTextFoundation() {
		set(true, RGB(0x00, 0x00, 0x00), RGB(0xFF, 0xFF, 0xFF));
	}
	void	set(bool bEnabled, COLORREF clrFore, COLORREF clrBg,
			bool bItalic = false, bool bBold = false, UnderlineType ultUnderline = ULT_NONE) {
		enabled = bEnabled;
		fgColor = clrFore;
		bgColor = clrBg;
		italic = bItalic;
		bold = bBold;
		underline = ultUnderline;
	}
};

///	CEditView::GetAllLineParams Ŏgp
struct TAppDefinedLine {
	/// _s
	unsigned long	iLine;
	/// AvP[V`l
	DWORD			dwParam;
};


/// r[̎GȐݒ
typedef unsigned short	EditViewOption;
/// OLE hbOAhhbvg
const EditViewOption	EVO_USE_OLEDRAGDROP			= 0x0001;
///	ݍsɉ\
const EditViewOption	EVO_SHOW_CURRENTUNDERLINE	= 0x0002;
///	s}[N\
const EditViewOption	EVO_SHOW_BREAKARROWS		= 0x0004;
///	EOF }[N\
const EditViewOption	EVO_SHOW_EOFMARK			= 0x0008;
///	󔒗ޕ\
const EditViewOption	EVO_SHOW_WHITESPACE			= 0x0010;
/// Unicode 䕶\ ()
const EditViewOption	EVO_SHOW_UNICODECTRLCHARS	= 0x0020;
///	IɑĂs̔wih
const EditViewOption	EVO_PAINTBREAKCONTINUINGSEL	= 0x0040;
///	tHggpɕԊu1
const EditViewOption	EVO_MAKECLOSEBOLDCHARS		= 0x0080;
///	\P⊮ő啶ʂȂ
const EditViewOption	EVO_IGNORECASE				= 0x0100;
///	L[{[hŃLbgړłȂ̑OɃ}EXňړł悤ɂ
const EditViewOption	EVO_ACCEPTABLECARETBYMOUSE	= 0x0200;
///	g[NPʂŃZbg
const EditViewOption	EVO_RESETDIRECTIONBYTOKEN	= 0x0400;


///	CEditView ̐ݒ荀
struct TEditViewOption {
	///	Cfg̎
	AutoIndentType	aitAutoIndentType;
	///	X}[gCfgΊʌōls
	unsigned long	cRecognizingLines;
	///	^ȗ֕
	wchar_t			chTabAlternative;
	///	̑̋󔒗ޕ̑֕
	wchar_t			chWhitespaceAlternative;
	///	EOF }[N̕
	std::wstring	strEndOfFile;
	///	P\ƂĈ̏W
	std::wstring	strCharsTreatedAsAlpha;
	///	̑̐ݒ
	EditViewOption	evoMiscellaneous;
};


// IEditViewEventListener interface definition
/////////////////////////////////////////////////////////////////////////////

///	CEditView CX^X̃Cxgnh
interface IEditViewEventListener {
	///	fXgN^
	virtual ~IEditViewEventListener() {
	}
	/**
	 *	sύXꂽ
	 *	@param iLine	ύXꂽ擪s (_s)
	 *	@param cLines	ύXꂽs
	 */
	virtual void	OnChangeLines(unsigned long iLine, unsigned long cLines) = 0;
	/**
	 *	s쐬ꂽ
	 *	@param iLine	쐬ꂽ擪s (_s)
	 *	@param cLines	쐬ꂽs
	 */
	virtual void	OnCreateLines(unsigned long iLine, unsigned long cLines) = 0;
	/**
	 *	s폜ꂽ
	 *	@param iLine	폜ꂽ擪s (_s)
	 *	@param cLines	폜ꂽs
	 */
	virtual void	OnDeleteLines(unsigned long iLine, unsigned long cLines) = 0;
	/**
	 *	уCWP[^}[W`ɌĂяo
	 *	@param iLine			_s
	 *	@param oDC				foCXReLXgB<strong>߂Oɑɖ߂</strong>
	 *	@param rectMargin		`͈
	 *	@param bBookmarked		̍sɃubN}[Nݒ肳Ă邩
	 *	@param dwParam			̍s̃AvP[V`l
	 *	@param bSingleEmphasis	̍sB̋\s
	 */
	virtual void	OnDrawIndicatorMargin(unsigned long iLine, CMinimalDC& oDC,
						const RECT& rectMargin, bool bBookmarked, DWORD dwParam, bool bSingleEmphasis) = 0;
	/**
	 *	Lbgړ
	 *	@param pos	ړ̃Lbgʒu
	 */
	virtual void	OnMoveCaret(const CCharPos& pos) = 0;
	/**
	 *	}EXJ[\ړ
	 *	@param pt	NCAgW
	 */
	virtual void	OnMoveCursor(const POINT& pt) = 0;
	/**
	 *	s`OɑOiFƔwiF₢킹
	 *	@param iLine			_s
	 *	@param bBookmarked		̍sɃubN}[Nݒ肳Ă邩
	 *	@param dwParam			̍s̃AvP[V`l
	 *	@param bSingleEmphasis	̍sB̋\s
	 *	@param clrFore			OiFB-1ԂƒʏʂF (Ăяoɂ-1ZbgĂ)
	 *	@param clrBack			wiFB-1ԂƒʏʂF (Ăяoɂ-1ZbgĂ)
	 */
	virtual void	OnQueryLineColor(unsigned long iLine, bool bBookmarked, DWORD dwParam,
						bool bSingleEmphasis, COLORREF& clrFore, COLORREF& clrBack) = 0;
};


// CClipboardRing class definition
/////////////////////////////////////////////////////////////////////////////

///	1̃Nbv{[heLXg\\
struct TClipText {
	///	eLXg
	std::wstring	strText;
	///	`f[^
	bool			bBox;
};

///	Nbv{[hOCxg
interface IClipboardRingEventListener {
	///	fXgN^
	virtual ~IClipboardRingEventListener() {}
	///	eLXg̃XgύXꂽ
	virtual void	OnClipboardRingChanged() = 0;
	///	CClipboardRing::Add \bhŒǉ悤ƂeLXg傫߁A󂯓ꂪۂꂽ
	virtual void	OnClipboardRingDeniedAdding() = 0;
};

///	Nbv{[hO
class CClipboardRing : public Manah::CObject {
	// RXgN^
public:
	CClipboardRing();

	// \bh
public:
	void			Add(const std::wstring& strText, bool bBox);
	void			Delete(unsigned char iText) throw(out_of_range);
	void			DeleteAll();
	unsigned char	GetCount() const;
	void			GetText(unsigned char iText, std::wstring& strText, bool& bBox) const throw(out_of_range);
	void			LimitCount(unsigned char nLimit);
	bool			Load(const std::wstring& strPathName);
	bool			Save(const std::wstring& strPathName) const;
	void			SetEventListener(IClipboardRingEventListener* pEventListener);

	// f[^o
private:
	std::list<TClipText>			m_listDatas;
	unsigned char					m_nLimit;
	unsigned long					m_nMaxByte;
	IClipboardRingEventListener*	m_pEventListener;
};


// CEditView class definition
/////////////////////////////////////////////////////////////////////////////

/**
 *	@brief	Ascension eLXgGfB^̃r[
 *
 *	<h3>KvpA</h3>
 *
 *	<ul>
 *		<li>̃NXł OLE hbOAhhbvgp̂ŁA
 *	gݍ݃AvP[VXbh OleInitialze API ĂяoĂKvB</li>
 *		<li>̓\yуC^[tFCX͑S Unicode gp邽߁A
 *	 Windows NT n݂̂ƂȂB</li>
 *		<li>̃NX͌_ł̓XbhZ[tł<strong>Ȃ</strong>B</li>
 *		<li> Unicode 4.0 ̒`ɂ͏]ĂȂB</li>
 *	</ul>
 *
 *	eLXgGfB^ƂĎgꍇ CEditView ACEditDoc CX^X𒼐ڂA
 *	CEditController gق悢
 *
 *	<h3>p</h3>
 *
 *	̃NXŎgpP̒`͈ȉ̂Ƃ:
 *	<dl>
 *		<dt>AJ[|Cg</dt><dd>IJn_BLbg</dd>
 *		<dt>ANeBu|Cg</dt><dd>II_BLbg</dd>
 *		<dt>P (Char)</dt><dd>Ch11P</dd>
 *		<dt>P (Column)</dt><dd>Œ蕝tHg̔p11P</dd>
 *		<dt>XN[W</dt><dd>XN[̍オ_</dd>
 *		<dt>NCAgW (Pos)</dt><dd>CEditView EBhE̍オ_</dd>
 *		<dt>r[W</dt><dd>NCAgW + (ѕ + ], э + ])BXN[͖</dd>
 *	</dl>
 *
 *	<h3>Rs[RXgN^ɂ镡ɂ (To deriver)</h3>
 *
 *	CEditView ̑cNX͑SăRs[RXgN^ĂA
 *	̃NXłRs[RXgN^găr[𕡐邱Ƃo (
 *	cNXƓlEBhE͍쐬Ȃ)Bf[^oɑ
 *	̃NX͈ꕔ̃o𕡐ƕŋL
 *
 *	̓Rs[RXgN^̉EӂŁALf[^o<strong>L</strong>B
 *	LȂr[폜ĂLf[^͔jȂA
 *	Lr[폜ƁAL͂̃r[畡ꂽ̃r[ɈڂB
 *	čŌɎcr[폜ꂽƂALf[^͔j󂳂B
 *	̂߃Rs[RXgN^ł͕̃r[A
 *	̃r[IWĩr[LB܂A
 *	r[͎j󂳂ꂽƂɃIWiɒʒm
 *
 *	܂ALێr[͎ɑ΂ݒ̍XV𑼂̃r[ɒʒmƂڂB
 *	NCAgr[̖{IłȂݒ (tHgsԊuAAvP[V`sȂ)
 *	Lr[ɑ΂čsƁAIɕ̃r[ɂf̂ŁA
 *	NCAg͂֘ASẴr[ݒ肷Kv͖
 *
 *	CEditController NXŕsꍇ2̃r[͂̕[ɏ]B
 *	L̂͊{Iɍŏ̃r[ANX GetPrimaryView
 *	\bhŊmɏLr[擾ł
 *
 *	<h3>`̓Ɖ</h3>
 *
 *	܂Ƃ܂sۂɑSĂ̒iKŕ`sĂƓ삪xȂ̂ŁA
 *	CEditView ͕`̓s߂ CEditView::Freeze \bhA
 *	邽߂ CEditView::Unfreeze \bh2pӂĂB
 *	ACEditView::InvalidateLine ACEditView::InvalidateLines
 *	͑ɂ͍ĕ`s킸A`̕KvȍsL^ĂA
 *	𓀂ƂɈxɕ`sB̂ CEditView ̂قƂǂ̑
 *	CWindow::InvalidateRect 𒼐ڌĂяôł͂ȂAL2̃\bhōĕ`s
 *
 *	֐̂߂ɓƂ̉s\bh͎̕A
 *	/s (J CEditView::_Unfreeze g)
 *
 *	@see	CEditDoc, CEditController
 */
class CEditView :
		public Manah::Windows::CView,
		public Armaiti::IUnknownImpl<Armaiti::CUnheapMemoryManageModel>,
		public IDropSource,
		public IDropTarget {

	///	{^ǂ̂悤ȖړIŉĂ邩\
	enum LeftDownMode {
		LDM_NONE,				// ĂȂ
		LDM_SELECTION,			// I͈͂̊g/k
		LDM_BOXSELECTION,		// `I͈͂̊g/k
		LDM_LINESELECTION,		// sI
		LDM_DRAGANDDROP,		// vZX̃hbOAhhbv
		LDM_DRAGANDDROPSELF,	// vZX̃hbOAhhbv
		LDM_DRAGANDDROPBOXSELF	// vZX̃hbOAhhbv (`)
	};

	friend class CEditController;
	friend class CLineLayoutManager;
	friend class CEditPoint;
	friend class CBoundarySearcher;
	friend class CAutoCompleteWnd;

	// f[^o
private:
	static bool								m_bRegistered;		// EBhENX̓o^tO
	static CClipboardRing					m_oClipboardRing;	// Nbv{[hO
	static CTextSearcher					m_oTextSearcher;	// eLXg
	static CPopupMenu						m_oContextMenu;		// ftHg̃ReLXgj[
	CBoundarySearcher*						m_pBoundarySearcher;// eLXgĚ
	Armaiti::OLE::CDraggingTextDataObject*	m_pDragging;		// OLE hbOIuWFNg
	CKeyMacroPlayer*						m_pKeyMacroPlayer;	// L[}N̊Ǘ
	CAutoCompleteWnd*						m_pwndAutoComplete;	// ͕⊮EBhE
	IEditViewEventListener*					m_pEventListener;	// CxgXi
	std::set<CEditPoint*>					m_setEditPoints;	// ێĂҏW_

	/* LÂ̊Ǘ (NX̐Q) */
	CEditView*		m_pOriginalView;		//  (this łΎ)
	std::set<CEditView*>	m_setClones;	// ̕

	/* CAEg */
	struct TEditViewLayout {	// CAEg܂Ƃ߂邾̍\
		unsigned int		nLineHeight;			// s̍ (ʁA)
		unsigned int		nCharHeight;			// ̍
		unsigned int		nCharWidth;				// p ()  (m_nLineHeight  API ŎIɎZo)
		int					nCharSpan;				// ̊Ԋu
		unsigned short		nTabWidth;				// ^u (pP)
		unsigned int		nLeftMargin;			// ]
		unsigned int		nTopMargin;				// ]
		unsigned int		nLeftTabWidth;			//  (sԍubN}[N\邽߂) ^uɕKvȒ
		unsigned int		nHScrollRatio;			// XN[1r[̉ɑ邩 (ʂ1)
		unsigned int		nVScrollRatio;			// XN[1sr[̉sɑ邩 (ʂ1)
		RECT				rectUpdate;				// OnPaint ŗLȍXV̈
		CCharPos			posHilightedBrackets[2];// \Ίʂ̈ʒu (ꍇ CCharPos(-1, -1))
		std::wstring		strFontFace;			// ̃tHg
		TTextFoundation		tfTextFoundation[TEXTTYPE_NUMBER];	// eLXg̐FȂ
		COLORREF			clrUnderline;			// ݍsɕ\鉺̐F
		COLORREF			clrIAUnderline;			// ANeBuɌݍsɕ\鉺̐F
		CLineLayoutInfo*	pAppDefinedSingleLine;	// AvP[V`s (Ps)
		unsigned long		iBoxSelectionAnchorLine;// `ĨAJ[|Cg̕\s
		unsigned long		iBoxSelectionActiveLine;// `ĨANeBu|Cg̕\s
		unsigned long		xBoxSelectionAnchor;	// `ĨAJ[|Cg̃r[ x W
		unsigned long		xBoxSelectionActive;	// `ĨANeBu|Cg̃r[ x W

		TEditViewLayout() : nLineHeight(13), nCharHeight(12), nCharWidth(7),
				nCharSpan(0), nTabWidth(4),
				nLeftMargin(1), nTopMargin(1), nHScrollRatio(1), nVScrollRatio(1),
				clrUnderline(-1), clrIAUnderline(-1), pAppDefinedSingleLine(0) {
					posHilightedBrackets[0] =
					posHilightedBrackets[1] = CCharPos(-1, -1);
		}
	} m_layoutInfo;

	CLineLayoutManager*	m_pLineLayoutManager;	// ܂ԂȂǂ̍sCAEg

	/* [h */
	struct TEditViewModeState {
		bool			bOvertype;			// ㏑[h
		bool			bRtlReading;		// E獶ɓǂނǂ
		bool			bResetDirByToken;	// g[NƂɕZbg邩ǂ
		bool			bShowUnicodeCtrls;	// Unicode 䕶\邩ǂ ()
		bool			bBoxSelecting;		// `I ({^ĂȂĂ悢)
		bool			bCaretMarkByMouse;	// Lbg󂯎Ȃ̑OɃ}EXňړł
		LeftTabType		lttTabType;			// LeftTabType `Q
		WrapMode		wpmWrapMode;		// ݂̐܂Ԃ[h (WordWrapMode `Q)
		unsigned long	nWrapWidth;			// w蕝Ő܂ԂƂ1s̕\ (ϕ*)
		bool			bUseOLEDragDrop;	// OLE hbOAhhbvgǂ
		AutoIndentType	aitAutoIndentType;	// Cfg̎
		unsigned long	cFindLimit;			// X}[gCfggpɉsO܂ł̊ʂɑΉ邩

		TEditViewModeState() : bOvertype(false), bRtlReading(false),
			bResetDirByToken(true), bShowUnicodeCtrls(false), bBoxSelecting(false),
			bCaretMarkByMouse(false), lttTabType(LTT_LINENUMBER), wpmWrapMode(WPM_NONE), nWrapWidth(100),
			bUseOLEDragDrop(true), aitAutoIndentType(AIT_BLOCK), cFindLimit(100) {
		}
	} m_ModeState;

	/* ⊮ */
	struct TAutoComplete {
		unsigned long	iCurrentLine;		// ͌EBhE\̍s
		unsigned long	iStartWordStart;	// ͌EBhE\OɂP̐擪ʒu
		unsigned long	iStartChar;			// ͌EBhE\̃Lbg̕ʒu
	} m_AutoComplete;

	bool			m_bActiveIMEComposition;	// IME œ͒
	LeftDownMode	m_ldmLeftDown;				// LeftDownMode `Q
	NextCharConvert	m_nccNextCharConvert;		// NextCharConvert `Q
	TEditOperation	m_eoLast;					// OɍsҏW
	bool			m_bFreezed;					// t[Y
	std::set<ulong>	m_setInvalidLines;			// t[Yɍĕ`vꂽsB
												// InvalidateLine(s)  OnUpdate Ǘ

	/* ݈ʒu */
	CVisibleEditPoint*	m_pAnchorPoint;	// IJnʒu (AJ[|Cg)
	CVisibleEditPoint*	m_pActivePoint;	// IIʒu (ANeBu|CgALbgʒu)
	POINT			m_ptScroll;			// XN[o[̈ʒu (x:Ay:)B
										// ̒l m_nHScrollRatio  m_nVScrollRatio 悸
										// \Jnʒu (:ϕPʁA:\sP) 
	unsigned long	m_iFirstVisibleLine;// ݂̐擪s (_sB\s m_ptScroll.y)B
										// _s<->\sϊ̂߂̃LbV
										// cXN[ (OnVScroll)A܂Ԃ񂪕ωƂ (OnSize,
										// ModifyFontInfo)A\Jn_sO̍sҏWꂽƂ
										// (OnUpdate) XV <- !݂͎gĂȂ!

	/* \ */
	typedef map<
		wchar_t, list<wstring>
	>	OperatorMap;
	struct TEditViewFoundationInfo {
		bool			bDrawCurrentUnderline;	// ݍs̉`悷邩ǂ
		bool			bDrawBreakArrow;		// s̖`悷邩ǂ
		bool			bDrawEOF;				// [EOF]`悷邩ǂ
		bool			bDrawSpaces;			// 󔒂̑֕`悷邩ǂ
		bool			bIgnoreCaseOnHilite;	// \RgJn/I񌟍ɑ啶ʂȂ
		bool			bPaintBreakContinuingSelection;	///	IɑĂs̔wih邩ǂ
		bool			bDrawBoldCharsClosely;	// l߂ĕ`悷邩ǂ
		wchar_t			chTab;					// ^ȗ֕
		wchar_t			chIdeographicSpace;		// Sp󔒂̑֕
		wchar_t			chSpace;				// ̑̋󔒗ޕ̑֕
		std::wstring	strEndOfFile;			// [EOF] }[N̕
		std::wstring	strCommentStart1;		// sRgJn1
		std::wstring	strCommentEnd1;			// sRgI1
		std::wstring	strCommentStart2;		// sRgJn2
		std::wstring	strCommentEnd2;			// sRgI2
		std::wstring	strCommentLineHeader1;	// PsRgJn1
		std::wstring	strCommentLineHeader2;	// PsRgJn2
		CKeywordManager*	pKeywords1;				// \L[[h1̃Xg
		CKeywordManager*	pKeywords2;				// \L[[h2̃Xg
		OperatorMap			mapOperators;			// Zq
		std::list<ulong>	listCharsTreatedAsAlpha;// L[[h\ƂĎgXg
		bool				bLockedParsing;			// s͂}Ă邩 (LockParsing Q)

		TEditViewFoundationInfo();
		TEditViewFoundationInfo(const TEditViewFoundationInfo& rhs);
		~TEditViewFoundationInfo();
	} m_FoundationInfo;

	/* GDI IuWFNg */
	struct TEditViewGDIObjects {
		Manah::Windows::GDI::CMinimalDC	oMemDC;	// EBhE DC Ɍ݊̂郁 DC (CEditView::Create 1񂾂쐬)
		HBITMAP	hMemLineBMP;	// 1s`̃rbg}bv (OnSize Ŕj/쐬)
		HBITMAP	hMemLeftTabBMP;	// т̃rbg}bv (SetLeftTabType AOnSize Ŕj/쐬)
		HPEN	hUnderlinePen;	// ݍs̉
		HPEN	hIAUnderlinePen;// ݍs̉ (ANeBu)
		HPEN	hLeftIndicatorPen;	// LTT_INDICATOR [hɍт`̂Ɏgp
		HPEN	hLineNumberPen;	// LTT_LINENUMBER [hɍт`̂Ɏgp
		HPEN	hColumnPen;		// TTT_COLUMNNUMBER [hɏт`̂Ɏgp
		HPEN	hWhiteSpacePen;	// 󔒗ޕ (̑փrbg}bv) `̂Ɏgp
		HPEN	hLineBreakPen;	// s}[N`̂Ɏgp
		HFONT	hNormalFont;	// ̖WtHgBvZɂgp
		HFONT	hBoldFont;		// `ɎgptHg
		HFONT	hItalicFont;	// C^bN̕`ɎgptHg
		HFONT	hBoldItalicFont;// C^bN̕`ɎgptHg

		TEditViewGDIObjects() : hMemLineBMP(0), hMemLeftTabBMP(0),
			hUnderlinePen(0), hIAUnderlinePen(0), hLeftIndicatorPen(0), hLineNumberPen(0), hColumnPen(0),
			hWhiteSpacePen(0), hLineBreakPen(0),
			hNormalFont(0), hBoldFont(0), hItalicFont(0), hBoldItalicFont(0) {
		}
		~TEditViewGDIObjects() {
			::DeleteObject(hMemLineBMP);
			::DeleteObject(hMemLeftTabBMP);
			::DeleteObject(hUnderlinePen);
			::DeleteObject(hIAUnderlinePen);
			::DeleteObject(hLeftIndicatorPen);
			::DeleteObject(hLineNumberPen);
			::DeleteObject(hColumnPen);
			::DeleteObject(hWhiteSpacePen);
			::DeleteObject(hLineBreakPen);
			::DeleteObject(hNormalFont);
			::DeleteObject(hBoldFont);
			::DeleteObject(hItalicFont);
			::DeleteObject(hBoldItalicFont);
		}
	} m_GDIObjects;

protected:
	static LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
	static LRESULT CALLBACK CandidateWordListProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);

	// RXgN^
public:
	CEditView();
	CEditView(const CEditView& rhs);
	virtual ~CEditView();

	// Zq
private:
	CEditView& operator =(const CEditView& rhs);	/// gpȂ

	// \bh
public:
	// 쐬
	virtual bool	Create(HWND hwndParent, const RECT* pRect,						// EBhE쐬
						DWORD dwStyle, DWORD dwExStyle) throw(runtime_error);

	// ̎擾
	void			GetAllLineParams(std::list<TAppDefinedLine>& listLines) const;	// AvP[V`l̑S擾
	DWORD			GetLineParam(unsigned long iLine) const;						// AvP[V`l̎擾
	void			GetOption(TEditViewOption& evo) const;							// GȐݒ̎擾
	unsigned short	GetTabWidth() const;											// ^u̎擾
	bool			HasRtlTextDirection() const;

	// ̐ݒ
	void	ModifyFontInfo(const LOGFONTW& lf, bool bRedraw = true);		// tHg̕ύX
	void	SetAppDefinedLine(unsigned long iLine);							// AvP[V`s̐ݒ
	void	SetCandidateWords(const std::list<TCandidateWord>& listWords);	// ͌PꃊXg̐ݒ
	void	SetCharSpaces(unsigned int nLineSpan, unsigned int nCharSpan);	// sÅԊu̕ύX
	void	SetTextDirection(bool bRtlReading);
	void	SetEventListener(IEditViewEventListener* pEventListener);		// CxgXi̓o^
	void	SetLeftTabType(LeftTabType ltt);								// [̑т̐ݒ
	void	SetLineParam(unsigned long iLine,								// AvP[V`l̐ݒ
				DWORD dwParam, DWORD dwMask = 0xFFFFFFFF);
	void	SetMargins(unsigned int nLeft, unsigned int nTop);				// ]̐ݒ
	void	SetTabWidth(unsigned short nTabWidth);							// ^u̐ݒ
	void	SetOption(TEditViewOption& evo);								// ̑̐ݒ

	// \ڂ̎擾
	TTextFoundation	GetTextFoundation(TextType type) const throw(out_of_range);		// C̎擾

	// \ڂ̐ݒ
	void	LockParsing();													// s͂}
	void	SetKeywordLists(const std::list<std::wstring>& listKeywords1,	// \2̐ݒ
				const std::list<std::wstring>& listKeywords2);
	void	SetMultilineCommentDelimiters(									// ubNRg̐ݒ
				const std::wstring& strBegin1, const std::wstring& strEnd1,
				const std::wstring& strBegin2, const std::wstring& strEnd2);
	void	SetOperators(const std::list<std::wstring>& listOperators);		// Zq̐ݒ
	void	SetSinglelineCommentHeaders(									// sRg̐ݒ
				const std::wstring& strHeader1, const std::wstring& strHeader2);
	void	SetTextFoundation(TextType type,								// C̐ݒ
				const TTextFoundation& tf) throw(out_of_range);
	void	UnlockParsing();												// s̗͂}

	// Lbg
	void		BeginBoxSelect();
	void		EndBoxSelect();
	CCharPos	GetActivePoint() const;
	CCharPos	GetAnchorPoint() const;
	void		GetSel(CCharPos& posAnchor, CCharPos& posActive) const;			// I͈͂̎擾
	bool		HasSelection() const;											// I͈͂邩ǂ
	void		RecreateCaret();												// Lbg쐬
	void		SetSel(const CCharPos& posAnchor, const CCharPos& posActive,	// I͈͂̐ݒ
					bool bScroll = true, bool bRepaint = true);
	void		SetSelWithoutSelection(const CCharPos& pos,						// Lbgʒu̐ݒ
					bool bScroll = true, bool bRepaint = true);
	void		SetSelWithoutSelection(unsigned int iLine, unsigned int iChar,
					bool bScroll = true, bool bRepaint = true);

	// ƒu
	unsigned long	BookmarkAll(const std::wstring& strText, SearchFlag sfFlag);	// Să}[N
	bool			FindNext(const std::wstring& strText, SearchFlag sfFlags);		// /O
	unsigned long	ReplaceAll(const std::wstring& strFindWhat,						// SĒu
						const std::wstring& strReplaceWith, SearchFlag sfFlag);
	bool			ReplaceAndFindNext(const std::wstring& strFindWhat,				// uĎ
						const std::wstring& strReplaceWith, SearchFlag sfFlag);

	// eLXg
	static CClipboardRing&	GetClipboardRing();								// Nbv{[hO擾
	std::wstring			GetNearestWordFromCaret() const;				// Lbgt߂̒P擾
	std::wstring			GetNearestWordFromCursor() const;				// J[\t߂̒P擾
	std::wstring			GetPrecedingWord(unsigned long cchLimit) const;	// Lbg̒OɂP擾
	std::wstring			GetSelection() const;							// I͈͂̎̕擾

	// ҏW_
	CEditPoint*	CreateEditPoint();

	// ubN}[N
	void	ClearBookmarks();												// ubN}[NSĉ
	void	GetBookmarkList(std::list<unsigned long>& listLines) const;		// ubN}[ÑXgԂ
	bool	HasBookmarkAt(unsigned long iLine) const throw(out_of_range);	// ubN}[Nݒ肳Ă邩Ԃ
	void	SetBookmark(unsigned long iLine,								// ubN}[N̐ݒ/
				bool bOn = true) throw(out_of_range);
	void	ToggleBookmark(unsigned long iLine) throw(out_of_range);		// ubN}[N̐ݒ/

	// `̓A
	void	Freeze();
	void	Unfreeze();

	// R}h
	void	ExecCommand(CommandIdentifier cmd, LPARAM lParam = 0L);	// R}ḧꊇ
protected:
	/* ړƑI */
	void	CmdMoveCancelSelection();
	void	CmdMoveCharNext(bool bExtend = false, unsigned long cch = 1);
	void	CmdMoveCharPrev(bool bExtend = false, unsigned long cch = 1);
	void	CmdMoveEnd(bool bExtend = false);
	void	CmdMoveHome(bool bExtend = false);
	void	CmdMoveLineDown(bool bExtend = false, unsigned long cLines = 1);
	void	CmdMoveLineEnd(bool bExtend = false);
	void	CmdMoveLineHome(bool bExtend = false);
	void	CmdMoveLineUp(bool bExtend = false, unsigned long cLines = 1);
	void	CmdMoveMatchBracket(bool bExtend = false);
	void	CmdMoveNextBookmark();
	void	CmdMovePageDown(bool bExtend = false, unsigned long cPages = 1);
	void	CmdMovePageUp(bool bExtend = false, unsigned long cPages = 1);
	void	CmdMovePrevBookmark();
	void	CmdMoveSelectAll();
	void	CmdMoveSelectCurrentWord();
	void	CmdMoveWordEndNext(bool bExtend = false, unsigned long cWords = 1);
	void	CmdMoveWordEndPrev(bool bExtend = false, unsigned long cWords = 1);
	void	CmdMoveWordNext(bool bExtend = false, unsigned long cWords = 1);
	void	CmdMoveWordPrev(bool bExtend = false, unsigned long cWords = 1);
	/* ҏW */
	void	CmdEditChar(unsigned long nCodePoint);
	void	CmdEditBreak();
	void	CmdEditInsertPrevLine();
	void	CmdEditBackspace();
	void	CmdEditDelete();
	void	CmdEditDeleteLine();
	void	CmdEditDeleteToNextWord();
	void	CmdEditDeleteToPrevWord();
	void	CmdEditUndo();
	void	CmdEditRedo();
	void	CmdEditConvertSel(SelConvertType sct);
	void	CmdEditCut();
	void	CmdEditCopy();
	void	CmdEditPaste();
	void	CmdEditSpaceIndent(bool bReverse = false, unsigned short nLevel = 1);
	void	CmdEditTabify(bool bRevoke = false);
	void	CmdEditTabIndent(bool bReverse = false, unsigned short nLevel = 1);
	void	CmdEditText(const wchar_t* pwszText);
	void	CmdEditToggleCharCodePoint();
	void	CmdEditToggleOvertypeMode();
	void	CmdEditSetNextInputConvert(NextCharConvert n);
	void	CmdEditToggleIMEStatus();
	void	CmdEditRecomposeSelection();
	void	CmdEditOpenCandidateWindow();
	/* XN[ */
	void	CmdScrollColumnNext();
	void	CmdScrollColumnPrev();
	void	CmdScrollEnd();
	void	CmdScrollEnsureCaretCentered();
	void	CmdScrollEnsureCaretVisible();
	void	CmdScrollHome();
	void	CmdScrollLineDown();
	void	CmdScrollLineUp();
	void	CmdScrollPageDown();
	void	CmdScrollPageUp();
	/* L[}N */
public:
	void	CmdKeymacroPlay();									// L[}N̎s
	void	CmdKeymacroRecord();								// L[}N̋L^
	void	CmdKeymacroStop();									// L[}N̋L^~
	void	CmdKeymacroPause();									// L[}N̋L^ꎞ~/ĊJ
	void	CmdKeymacroLoad(const std::wstring& strPathName);	// L[}N̓ǂݍ
	void	CmdKeymacroSave(const std::wstring& strPathName);	// L[}N̕ۑ
	void	CmdKeymacroCancel();								// L[}N̋L^~
	/* ̑̓ */
	bool	SmartIndent(wchar_t ch);						// X}[gCfgs

	// [eBeB
	CCharPos		CharFromPos(const POINT& pt,					// NCAgWƃXN[ԂsƕvZ
						bool bIgnoreMark, bool* bTruncated = 0) const;
	unsigned long	ColumnFromChar(const CCharPos& pos) const;					// ʒuԍ擾
	CCharPos		DisplayCharFromLogicalChar(const CCharPos& posLogical) const;	// _ʒu\ʒuɕϊ
	unsigned long	DisplayLineFromLogicalLine(unsigned long iLine) const;	// _s\sɕϊ
	unsigned long	GetDisplayLineCount() const;					// \sԂ
	void			GetDisplayLineOffsetIndex(unsigned long iDisplayLine,		// \s_sƃItZbgɕϊ
						unsigned long& iLogicalLine, unsigned long& iOffset) const;
	int				GetVisibleLineCount() const;					// EBhEƕ̍\\ȍsvZ
	int				GetVisibleCharCount() const;					// EBhEƕ̕\\1sɕvZ
	bool			IsOverSelection(const POINT& pt) const;						// ʒuI͈͏ォǂ
	bool			IsOverURIString(const POINT& pt, std::wstring& strURI) const;	// ʒu URI ォǂ
	CCharPos		LogicalCharFromDisplayChar(const CCharPos& posDisplay) const;	// \ʒu_ʒuɕϊ
	POINT			PosFromChar(const CCharPos& col) const;		// sƗAXN[ԂNCAgWvZ

	// Nbv{[h (ȊȎ HandleCommand gp)
	UINT	CanPaste() const;	// Nbv{[h̓ey[Xgł邩ǂԂ

	// hLgƂ̒ʐM
	virtual CEditDoc*	GetDocument() const;	// hLgIuWFNgւ̃ANZX

	// [h
	KeyMacroState		GetKeyMacroState() const;				// L[}N̏ԂԂ
	static CEditView*	GetViewKeyMacroRecording();				// L[}NL^/ꎞ~̃r[Ԃ
	WrapMode			GetWrapMode() const;					// ܂Ԃ[hԂ
	bool				IsOvertypeMode() const;					// ㏑[hǂԂ
	void				SetOvertypeMode(bool bOvertype = true);	// ㏑[h̐ݒ
	void				SetWrapMode(WrapMode wpm);				// ܂Ԃ[h̐ݒ
	void				SetWrapWidth(unsigned long nWrapWidth);	// ܂Ԃ̐ݒ

	// IUnknown C^[tFCX
	IMPLEMENT_UNKNOWN()
	BEGIN_INTERFACE_TABLE()
		IMPLEMENTS_LEFTMOST_INTERFACE(IDropSource)
		IMPLEMENTS_INTERFACE(IDropTarget)
//		IMPLEMENTS_INTERFACE(IDispatch)
//		IMPLEMENTS_INTERFACE(IViewObjectEx)
//		IMPLEMENTS_INTERFACE(IViewObject2)
//		IMPLEMENTS_INTERFACE(IViewObject)
//		IMPLEMENTS_INTERFACE(IOleInPlaceObjectWindowless)
//		IMPLEMENTS_INTERFACE(IOleInPlaceObject)
//		IMPLEMENTS_INTERFACE(IOleWindow, IOleInPlaceObjectWindowless)
//		IMPLEMENTS_INTERFACE(IOleInPlaceActiveObject)
//		IMPLEMENTS_INTERFACE(IOleControl)
//		IMPLEMENTS_INTERFACE(IOleObject)
//		IMPLEMENTS_INTERFACE(IPersistStreamInit)
//		IMPLEMENTS_INTERFACE(ISupportErrorInfo)
	END_INTERFACE_TABLE()

	// IDropSource C^[tFCX
	STDMETHODIMP	QueryContinueDrag(BOOL fEscapePressed, DWORD grfKeyState);
	STDMETHODIMP	GiveFeedback(DWORD dwEffect);

	// IDropTarget C^[tFCX
	STDMETHODIMP	DragEnter(IDataObject* pDataObj, DWORD grfKeyState, POINTL pt, DWORD* pdwEffect);
	STDMETHODIMP	DragOver(DWORD grfKeyState, POINTL pt, DWORD* pdwEffect);
	STDMETHODIMP	DragLeave();
	STDMETHODIMP	Drop(IDataObject* pDataObj, DWORD grfKeyState, POINTL pt, DWORD* pdwEffect);

	// J[eBeB (I͈̓eLXgwp)
protected:
	void	DeleteSel();															// I͈͂̍폜
	void	InsertText(const std::wstring& strText);								// Lbgʒuւ̑̕}
public:	// b...
	void	ReplaceSel(const std::wstring& strTextbool, bool bBoxPaste = false);	// I͈͂̒u

	// J[eBeB ()
protected:
	static unsigned long	IsNumeric(const wchar_t* psz, unsigned long cch);			// l
	static unsigned long	IsSingleQuotation(const wchar_t* psz, unsigned long cch);	// VONH[gň͂܂ꂽ
	static unsigned long	IsDoubleQuotation(const wchar_t* psz, unsigned long cch);	// _uNH[gň͂܂ꂽ
	static unsigned long	IsWhiteSpace(const wchar_t* psz, unsigned long cch);		// 󔒗ނ
	static unsigned long	IsURIString(const wchar_t* psz, unsigned long cch);			// URI 
	bool			CharIsInCommentOrQuotation(const CCharPos& pos) const;				// Rgp
	unsigned long	IsIdentifier(const wchar_t* psz, unsigned long cch) const;			// ʎq
	bool			IsSComment1(const wchar_t* psz, unsigned long cch) const;			// PsRg1T
	bool			IsSComment2(const wchar_t* psz, unsigned long cch) const;			// PsRg2T
	unsigned long	IsMComment1(const wchar_t* psz, unsigned long cch, bool& bContinued) const;	// sRg1T
	unsigned long	IsMComment2(const wchar_t* psz, unsigned long cch, bool& bContinued) const;	// sRg2T
	unsigned long	IsOperator(const wchar_t* psz, unsigned long cch) const;			// Zq
	unsigned char	IsInvalidCharacter(const wchar_t* psz, unsigned long cch) const;	// sȕ

	// J[eBeB ()
protected:
	static bool	IsIdentifierContinueChar(unsigned long nCodePoint);
	static bool	IsIdentifierStartChar(unsigned long nCodePoint);
				
	// J[eBeB (`)
protected:
	virtual int				DrawBreakMark(int x, int y, BreakType bt, COLORREF clrBack);	// s}[N̕`
	virtual void			DrawLeftTab(unsigned long iStart, unsigned long iEnd);			// т`
	virtual unsigned long	DrawLine(unsigned long iLine, int y,							// 1s`
								const std::wstring& strLine, BreakType bt, const CLineLayoutInfo* pLineInfo);
	HFONT					GetFontForRenderingToken(TextType type) const;					// w肵g[N`tHgԂ
	void					InvalidateLine(unsigned long iLine);							// ws`𖳌
	void					InvalidateLines(unsigned long iStart, unsigned long iEnd);		// wԍs`𖳌
private:
	void					_Unfreeze();

	// J[eBeB (ʒuvZ)
protected:
	wstring			CalculateSpacesReachingVirtualPoint(
						unsigned long iLine, unsigned long xVirtual) const;
	unsigned long	CharFromPixel(unsigned long iLine, unsigned long x,
						bool bIgnoreMark, bool* pbTruncated = 0) const;
	unsigned int	GetAvgCharWidth() const;										// 1̕ϕԂ
	unsigned long	GetNextTabStop(unsigned long xView, bool bRtlReading) const;	// ̃^uʒuԂ
	static int		MakeCaretPosValid(const wchar_t* pwsz,							// LbgʒȕC
						unsigned long cch, unsigned long nCaretPos, bool bBackward);

	// J[eBeB (I͈͂ƃLbg)
protected:
	void						CheckMatchBrackets();
	const CVisibleEditPoint&	GetSelTopPoint() const;
	const CVisibleEditPoint&	GetSelBottomPoint() const;
	virtual void				OnMoveCaret();

	// J[eBeB (̑)
protected:
	virtual int			CompareString(const wchar_t* psz1,
							const wchar_t* psz2, unsigned long cch) const;
	static wchar_t		ConvertCharacter(wchar_t wch, NextCharConvert ncc);
	virtual bool		FindBracket(const CCharPos& pos, CCharPos& posFound) const;
	unsigned int		GetLineNumberMaxDigit() const;
	void				InitializeWindow();
	bool				IsAcceptableAsMacroKey(UINT nChar) const;
	static bool			IsNeutralNoAlpha(wchar_t wch);
	void				ModifyScrollInfo(bool bHorizontal, bool bVertical);
	void				RecalcLeftTabWidth(int cy = -1);
	static bool			RegisterWindowClass();
	virtual void		UpdateGDIObjects();
	virtual void		ValidateCaretPos();
	void				ValidateIMEWindowPos();

	// J[eBeB (̑) ʂɌJ
public:
	virtual void	DrawUnderline(int x1, int x2, int y, UnderlineType ult);		// 

protected:
	virtual LRESULT	DispatchEvent(UINT message, WPARAM wParam, LPARAM lParam);
	void			ReleaseEditPoint(CEditPoint& pt);

	// bZ[Wnh
protected:
	virtual void	OnChar(UINT nChar, UINT nFlags);						// WM_CHAR
	virtual bool	OnCommand(WORD wID, WORD wNotifyCode, HWND hwndCtrl);	// WM_COMMAND
	virtual bool	OnContextMenu(HWND hWnd, POINT pt);						// WM_CONTEXTMENU
	virtual void	OnDestroy();											// WM_DESTROY
	virtual void	OnHScroll(UINT nSBCode, UINT nPos, HWND hwndScrollBar);	// WM_HSCROLL
	virtual bool	OnImeComposition(WPARAM wParam, LPARAM lParam);			// WM_IME_COMPOSITION
	virtual void	OnImeEndComposition();									// WM_IME_ENDCOMPOSITION
	virtual void	OnImeStartComposition();								// WM_IME_STARTCOMPOSITION
	virtual void	OnKeyDown(UINT nChar, UINT nFlags);						// WM_KEYDOWN
	virtual void	OnKillFocus(HWND hwndNew);								// WM_KILLFOCUS
	virtual void	OnLButtonDblClk(UINT nFlags, POINT pt);					// WM_LBUTTONDBLCLK
	virtual void	OnLButtonDown(UINT nFlags, POINT pt);					// WM_LBUTTONDOWN
	virtual void	OnLButtonUp(UINT nFlags, POINT pt);						// WM_LBUTTONUP
	virtual void	OnMouseMove(UINT nFlags, POINT pt);						// WM_MOUSEMOVE
	virtual bool	OnMouseWheel(UINT nFlags, short zDelta, POINT pt);		// WM_MOUSEWHEEL
	virtual void	OnPaint();												// WM_PAINT
	virtual bool	OnSetCursor(HWND hWnd, UINT nHitTest, UINT message);	// WM_SETCURSOR
	virtual void	OnSetFocus(HWND hwndOld);								// WM_SETFOCUS
	virtual void	OnSize(UINT nType, int cx, int cy);						// WM_SIZE
	virtual void	OnSysColorChange();										// WM_SYSCOLORCHANGE
	virtual void	OnTimer(UINT nIDEvent);									// WM_TIMER
	virtual void	OnVScroll(UINT nSBCode, UINT nPos, HWND hwndScrollBar);	// WM_VSCROLL

	// WbZ[Wnh
	virtual void	OnDocumentSetup(LPARAM lHint = 0);								// hLgA[hꂽ
	virtual void	OnUpdate(Manah::CObject* pSender, LPARAM lParam, void* pHint);	// hLg̒ʐM
};

// CC֐̎
/////////////////////////////////////////////////////////////////////////////

/**
 *	u\tv삪\擾
 *	@return	\t\ȃNbv{[h`Bs̏ꍇ0
 */
inline UINT CEditView::CanPaste() const {
	AssertValidAsWindow();

	if(GetDocument()->IsReadOnly())
		return 0;

	UINT	nBoxClipFormat = ::RegisterClipboardFormatW(L"MSDEVColumnSelect");

	if(nBoxClipFormat != 0 && ::IsClipboardFormatAvailable(nBoxClipFormat))
		return nBoxClipFormat;
	if(::IsClipboardFormatAvailable(CF_UNICODETEXT))
		return CF_UNICODETEXT;
	if(::IsClipboardFormatAvailable(CF_TEXT))
		return CF_TEXT;

	return 0;
}

///	wʒuRgptłΐ^Ԃ
inline bool CEditView::CharIsInCommentOrQuotation(const CCharPos& pos) const {
	AssertValid();

	const CLineLayoutInfo*		pLine = m_pLineLayoutManager->GetLine(pos.m_iLine);
	const TokenList&			listTokens = pLine->GetTokenList();
	TokenList::const_iterator	itTokens = listTokens.end();
	TextType					ttType;

	// pos s̏ꍇ
	ttType = (--itTokens)->GetType();
	if(ttType == TT_COMMENT) {
		const wstring&	strLine = GetDocument()->GetLine(pos.m_iLine);
		if(pos.m_iChar == strLine.length())
			return pLine->GetMultilineCommentStatus(true) != MCT_NOTCOMMENT;
	} else if(ttType == TT_DOUBLEQUOTATION || ttType == TT_SINGLEQUOTATION) {
		const wstring&	strLine = GetDocument()->GetLine(pos.m_iLine);
		if(pos.m_iChar == strLine.length()) {
			if(itTokens->GetStringLength() == 1)
				return true;
			return (ttType == TT_DOUBLEQUOTATION
					&& strLine[pos.m_iChar - 1] == L'\"'
					&& strLine[pos.m_iChar - 2] == L'\\')
				|| (ttType == TT_SINGLEQUOTATION
					&& strLine[pos.m_iChar - 1] == L'\''
					&& strLine[pos.m_iChar - 2] == L'\\');
		}
	}

	for(itTokens = listTokens.begin(); itTokens != listTokens.end(); ++itTokens) {
		if(pos.m_iChar > itTokens->GetIndex()
				&& pos.m_iChar < itTokens->GetIndex() + itTokens->GetStringLength()) {
			ttType = itTokens->GetType();
			return ttType == TT_COMMENT
				|| ttType == TT_DOUBLEQUOTATION
				|| ttType == TT_SINGLEQUOTATION;
		}
	}
	return false;
}

/**
 *	ݎgpĂtHg (C^bNłȂ) 
 *	1`悷Ƃ̕ϕԂ
 */
inline unsigned int CEditView::GetAvgCharWidth() const {
	AssertValid();
	return m_layoutInfo.nCharWidth + m_layoutInfo.nCharSpan;
}

///	Nbv{[hOԂ
inline CClipboardRing& CEditView::GetClipboardRing() {
	return m_oClipboardRing;
}

/**
 *	w肵g[N`tHgԂ
 *	@param type	g[N̎
 *	@return		tHg
 */
inline HFONT CEditView::GetFontForRenderingToken(TextType type) const {
	AssertValid();

	TTextFoundation	tf = m_layoutInfo.tfTextFoundation[type];
	if(tf.enabled && tf.bold)
		return tf.italic ? m_GDIObjects.hBoldItalicFont : m_GDIObjects.hBoldFont;
	else
		return (tf.enabled && tf.italic) ? m_GDIObjects.hItalicFont : m_GDIObjects.hNormalFont;
}

///	݂̃L[}N̏ԂԂ
inline KeyMacroState CEditView::GetKeyMacroState() const {
	AssertValid();
	return m_pKeyMacroPlayer->GetState();
}

/**
 *	hLǧ݂̍ŏIsl10iŉɂȂ邩Ԃ
 *	@return	Kvȕ
 */
inline unsigned int CEditView::GetLineNumberMaxDigit() const {
	AssertValid();

	unsigned int	n = 1;
	unsigned long	nLineCount = GetDocument()->GetLineCount();

	while(nLineCount >= 10) {
		nLineCount /= 10;
		++n;
	}
	return n;
}

/**
 *	̃^uʒuԂ
 *	@param xView		r[ x W
 *	@param bRtlReading	E獶ɓǂޏꍇ ()
 */
inline unsigned long CEditView::GetNextTabStop(unsigned long xView, bool bRtlReading) const {
	AssertValid();
	if(!bRtlReading)
		return xView + m_layoutInfo.nTabWidth * GetAvgCharWidth()
			- xView % (m_layoutInfo.nTabWidth * GetAvgCharWidth());
	else
		return xView - xView % (m_layoutInfo.nTabWidth * GetAvgCharWidth());
}

///	GȐݒԂ
inline void CEditView::GetOption(TEditViewOption& evo) const {
	AssertValid();

	evo.aitAutoIndentType = m_ModeState.aitAutoIndentType;
	evo.chTabAlternative = m_FoundationInfo.chTab;
	evo.chWhitespaceAlternative = m_FoundationInfo.chSpace;
	evo.cRecognizingLines = m_ModeState.cFindLimit;
	evo.strEndOfFile = m_FoundationInfo.strEndOfFile;

	EditViewOption&	misc = evo.evoMiscellaneous;
	misc = 0;
	if(m_FoundationInfo.bDrawBreakArrow)				misc |= EVO_SHOW_BREAKARROWS;
	if(m_FoundationInfo.bDrawCurrentUnderline)			misc |= EVO_SHOW_CURRENTUNDERLINE;
	if(m_FoundationInfo.bDrawEOF)						misc |= EVO_SHOW_EOFMARK;
	if(m_FoundationInfo.bDrawSpaces)					misc |= EVO_SHOW_WHITESPACE;
	if(m_FoundationInfo.bIgnoreCaseOnHilite)			misc |= EVO_IGNORECASE;
	if(m_FoundationInfo.bPaintBreakContinuingSelection)	misc |= EVO_PAINTBREAKCONTINUINGSEL;
	if(m_ModeState.bShowUnicodeCtrls)					misc |= EVO_SHOW_UNICODECTRLCHARS;
	if(m_ModeState.bUseOLEDragDrop)						misc |= EVO_USE_OLEDRAGDROP;

	std::wostringstream	ssChars;
	const list<ulong>&	listAlphas = m_FoundationInfo.listCharsTreatedAsAlpha;
	wchar_t				wszSurrogates[3] = {0, 0, 0};
	for(list<ulong>::const_iterator it = listAlphas.begin(); it != listAlphas.end(); ++it) {
		if(*it >= 0x010000) {
			Manah::Text::EncodeCodePointToUTF16SurrogatePair(*it, wszSurrogates);
			ssChars << wszSurrogates;
		} else
			ssChars << static_cast<wchar_t>(*it);
	}
	evo.strCharsTreatedAsAlpha = ssChars.str();
}

///	݂̃^uԂ
inline unsigned short CEditView::GetTabWidth() const {
	AssertValid();
	return m_layoutInfo.nTabWidth;
}

/**
 *	C̎擾
 *	@param type			̎
 *	@return				C
 *	@throw out_of_range	<var>type</var> sȂƂX[
 *	@see				SetTextFoundation
 */
inline TTextFoundation CEditView::GetTextFoundation(TextType type) const throw(out_of_range) {
	AssertValid();
	if(type < TT_NORMAL || type >= TEXTTYPE_NUMBER)
		throw out_of_range("Specified type is invalid.");

	TTextFoundation	tf = m_layoutInfo.tfTextFoundation[type];

	// OiF̎
	if(tf.fgColor == -1) {
		if(type == TT_NORMAL)					tf.fgColor = ::GetSysColor(COLOR_WINDOWTEXT);
		else if(type == TT_SELECTION)			tf.fgColor = ::GetSysColor(COLOR_HIGHLIGHTTEXT);
		else if(type == TT_INACTIVESELECTION)	tf.fgColor = ::GetSysColor(COLOR_INACTIVECAPTIONTEXT);
		else									tf.fgColor = (m_layoutInfo.tfTextFoundation[TT_NORMAL].fgColor != -1) ?
													m_layoutInfo.tfTextFoundation[TT_NORMAL].fgColor : ::GetSysColor(COLOR_WINDOWTEXT);
	}

	// wiF̎
	if(tf.bgColor == -1) {
		if(type == TT_NORMAL)					tf.bgColor = ::GetSysColor(COLOR_WINDOW);
		else if(type == TT_SELECTION)			tf.bgColor = ::GetSysColor(COLOR_HIGHLIGHT);
		else if(type == TT_INACTIVESELECTION)	tf.bgColor = ::GetSysColor(COLOR_INACTIVECAPTION);
		else									tf.bgColor = (m_layoutInfo.tfTextFoundation[TT_NORMAL].bgColor != -1) ?
													m_layoutInfo.tfTextFoundation[TT_NORMAL].bgColor : ::GetSysColor(COLOR_WINDOW);
	}

	return tf;
}

///	L[}NL^/ꎞ~̃r[Ԃ
inline CEditView* CEditView::GetViewKeyMacroRecording() {
	return CKeyMacroPlayer::Create()->GetActiveView();
}

/**
 *	NCAg̈ɕ\ł񐔂擾
 *	@return	\\ȗ
 */
inline int CEditView::GetVisibleCharCount() const {
	RECT	rect;
	GetClientRect(&rect);
	return (rect.right - rect.left == 0) ? 0 :
		(rect.right - rect.left - m_layoutInfo.nLeftMargin - m_layoutInfo.nLeftTabWidth) / GetAvgCharWidth();
}

/**
 *	NCAg̈ɕ\łs擾
 *	@return	\\ȍs
 */
inline int CEditView::GetVisibleLineCount() const {
	RECT	rect;
	GetClientRect(&rect);
	return (rect.bottom - rect.left == 0) ? 0:
		(rect.bottom - rect.top - m_layoutInfo.nTopMargin) / m_layoutInfo.nLineHeight;
}

///	܂Ԃ[hԂ
inline WrapMode CEditView::GetWrapMode() const {
	AssertValid();
	return m_ModeState.wpmWrapMode;
}

///	E獶ɓǂރ[h̏ꍇ true Ԃ
inline bool CEditView::HasRtlTextDirection() const {
	AssertValid();
	return m_ModeState.bRtlReading;
}

/**
 *	L[}NɕۑłL[ǂԂ
 *	@param nChar	zL[
 */
inline bool CEditView::IsAcceptableAsMacroKey(UINT nChar) const {
	switch(nChar) {
	case VK_LBUTTON:
	case VK_RBUTTON:
	case VK_CANCEL:
	case VK_MBUTTON:
	case VK_MENU:
	case VK_LWIN:
	case VK_RWIN:
	case VK_APPS:
	case VK_LMENU:
	case VK_RMENU:
		return false;
	}
	return true;
}

/**
 *	_uNH[gň͂܂ꂽ̒Ԃ (pɓ)B
 *	GXP[vɂ̓obNXbVgp
 *	@param psz	ׂ镶
 *	@param cch	
 *	@return		p̒BȂ0BĂȂ <var>cch</var> 𓯂lԂ
 */
inline unsigned long CEditView::IsDoubleQuotation(const wchar_t* psz, unsigned long cch) {
	assert(psz != 0);

	if(*psz != L'\"')
		return 0;
	for(unsigned long i = 1; i < cch; ++i) {
		if(*(psz + i) == L'\\')	// ͖̕
			++i; 
		else if(*(psz + i) == L'\"')
			return i + 1;
	}
	return cch;
}

/**
 *	ʎq̒Ԃ
 *	@param psz	ׂ镶
 *	@param cch	
 *	@return		ʎq\AĂ钷BʎqłȂ0
 *	@see		CEditView::IsIdentifierContinueChar,
 *				CEditView::IsIdentifierStartChar
 */
inline unsigned long CEditView::IsIdentifier(const wchar_t* psz, unsigned long cch) const {
	assert(psz != 0);

	using namespace Manah::Text;

	const std::list<unsigned long>&	listAlphas = m_FoundationInfo.listCharsTreatedAsAlpha;
	unsigned long					nCodePoint;
	for(unsigned long i = 0; i < cch; ++i) {
		if(IsUTF16HighSurrogate(psz[i])
				&& i < cch - 1
				&& IsUTF16LowSurrogate(psz[i + 1]))
			nCodePoint = DecodeUTF16SurrogatePairToCodePoint(psz + i, cch - i);
		else
			nCodePoint = psz[i];
		if(std::binary_search(listAlphas.begin(), listAlphas.end(), nCodePoint)
				|| (i == 0 && CEditView::IsIdentifierStartChar(nCodePoint))
				|| CEditView::IsIdentifierContinueChar(nCodePoint)) {
			if(nCodePoint >= 0x010000)
				++i;
			continue;
		}
		return i;
	}
	return cch;
}

/**
 *	݂̃R[hy[Wŕsȕ̒Ԃ
 *	@param psz	ׂ镶ւ̃|C^
 *	@param cch	<var>psz</var> ̒
 *	@return		sȕłȂꍇ0BsȒ U+10000 ̏ꍇ2BȊO1
 */
inline unsigned char CEditView::IsInvalidCharacter(const wchar_t* psz, unsigned long cch) const {
	assert(psz != 0);

	using namespace Manah::Text;

	if(cch == 0)
		return 0;
	else if(cch > 1
			&& IsUTF16HighSurrogate(psz[0])
			&& IsUTF16LowSurrogate(psz[1])) {
		BOOL	b = FALSE;
		::WideCharToMultiByte(GetDocument()->GetCodePage(), 0, psz, 2, 0, 0, 0, &b);
		return toBoolean(b) ? 1 : 0;
	} else {
		BOOL	b = FALSE;
		::WideCharToMultiByte(GetDocument()->GetCodePage(), 0, psz, 1, 0, 0, 0, &b);
		return toBoolean(b) ? 1 : 0;
	}
}

/**
 *	sRg1̒Ԃ (RgJnAIɓ)B
 *	@param psz			ׂ镶
 *	@param cch			<var>psz</var> ̒
 *	@param bContinued	ׂ镶񂪊ɕsRg1ł邱ƂĂ true w肷B
 *						̃\bh͏IɃRgIĂ false AłȂ true ݒ肷
 */
inline unsigned long CEditView::IsMComment1(const wchar_t* psz, unsigned long cch, bool& bContinued) const {
	assert(psz != 0);

	if(m_FoundationInfo.strCommentStart1.empty() || m_FoundationInfo.strCommentEnd1.empty())
		return 0;
	if(bContinued && cch < m_FoundationInfo.strCommentEnd1.length())
		return cch;
	if(!bContinued && cch < m_FoundationInfo.strCommentStart1.length())
		return 0;

	unsigned long	i = 0;

	if(!bContinued) {	// Jñ`FbN
		if(0 == CompareString(psz,
				m_FoundationInfo.strCommentStart1.c_str(),
				m_FoundationInfo.strCommentStart1.length())) {
			bContinued = true;
			i = m_FoundationInfo.strCommentStart1.length();
		} else
			return 0;
	}

	// IT
	if(m_FoundationInfo.bIgnoreCaseOnHilite) {	// 啶ʂȂꍇ
		wchar_t*	pszDummy = new wchar_t[cch];
		wchar_t*	pszEndStr = new wchar_t[m_FoundationInfo.strCommentEnd1.length() + 1];
		wcsncpy(pszDummy, psz, cch);
		Manah::Text::ToLower(pszDummy, cch);
		wcscpy(pszEndStr, m_FoundationInfo.strCommentEnd1.c_str());
		Manah::Text::ToLower(pszEndStr);
		std::wstring	str = wstring(pszDummy, cch);
		delete[] pszDummy;
		pszDummy = 0;
		i = str.find(pszEndStr, i);
		delete[] pszEndStr;
		pszEndStr = 0;
	} else {
		std::wstring	str = wstring(psz, cch);
		i = str.find(m_FoundationInfo.strCommentEnd1, i);
	}
	if(i == wstring::npos) {
		bContinued = true;
		return cch;
	} else {
		bContinued = false;
		return i + m_FoundationInfo.strCommentEnd1.length();
	}
}

/**
 *	sRg2̒Ԃ (RgJnAIɓ)B
 *	@param psz			ׂ镶
 *	@param cch			<var>psz</var> ̒
 *	@param bContinued	ׂ镶񂪊ɕsRg1ł邱ƂĂ true w肷B
 *						̃\bh͏IɃRgIĂ false AłȂ true ݒ肷
 */
inline unsigned long CEditView::IsMComment2(const wchar_t* psz, unsigned long cch, bool& bContinued) const {
	assert(psz != 0);

	if(m_FoundationInfo.strCommentStart2.empty() || m_FoundationInfo.strCommentEnd2.empty())
		return 0;
	if(bContinued && cch < m_FoundationInfo.strCommentEnd2.length())
		return cch;
	if(!bContinued && cch < m_FoundationInfo.strCommentStart2.length())
		return 0;

	unsigned long	i = 0;

	if(!bContinued) {	// Jñ`FbN
		if(0 == CompareString(psz,
				m_FoundationInfo.strCommentStart2.c_str(),
				m_FoundationInfo.strCommentStart2.length())) {
			bContinued = true;
			i = m_FoundationInfo.strCommentStart2.length();
		} else
			return 0;
	}

	// IT
	if(m_FoundationInfo.bIgnoreCaseOnHilite) {	// 啶ʂȂꍇ
		wchar_t*	pszDummy = new wchar_t[cch];
		wchar_t*	pszEndStr = new wchar_t[m_FoundationInfo.strCommentEnd2.length() + 1];
		wcsncpy(pszDummy, psz, cch);
		Manah::Text::ToLower(pszDummy, cch);
		wcscpy(pszEndStr, m_FoundationInfo.strCommentEnd2.c_str());
		Manah::Text::ToLower(pszEndStr);
		std::wstring	str = wstring(pszDummy, cch);
		delete[] pszDummy;
		pszDummy = 0;
		i = str.find(pszEndStr, i);
		delete[] pszEndStr;
		pszEndStr = 0;
	} else {
		std::wstring	str = wstring(psz, cch);
		i = str.find(m_FoundationInfo.strCommentEnd2, i);
	}
	if(i == wstring::npos) {
		bContinued = true;
		return cch;
	} else {
		bContinued = false;
		return i + m_FoundationInfo.strCommentEnd2.length();
	}
}

///	ԕ̔P\ǂԂ ([U`P\͖)
inline bool CEditView::IsNeutralNoAlpha(wchar_t wch) {
	WORD	wCharType;

	::GetStringTypeW(CT_CTYPE2, &wch, 1, &wCharType);
	if(wCharType >= C2_BLOCKSEPARATOR && wCharType <= C2_OTHERNEUTRAL) {
		::GetStringTypeW(CT_CTYPE3, &wch, 1, &wCharType);
		return !toBoolean(wCharType & (C3_DIACRITIC | C3_VOWELMARK | /*C3_LEXICAL |*/ C3_ALPHA));
	}
	return false;
}

///	l\̒ԂBlłȂ 0 Ԃ
inline unsigned long CEditView::IsNumeric(const wchar_t* psz, unsigned long cch) {
	assert(psz != 0);

	using namespace Manah::Text;

//	const std::list<unsigned long>&	listAlphas = m_FoundationInfo.listCharsTreatedAsAlpha;
	unsigned long					cp;

	if(!toBoolean(iswdigit(*psz)))
		return 0;
	for(unsigned long i = 1; i < cch; ++i) {
		if(i < cch - 1
				&& IsUTF16HighSurrogate(psz[i])
				&& IsUTF16LowSurrogate(psz[i + 1]))
			cp = DecodeUTF16SurrogatePairToCodePoint(psz + i, cch - i);
		else
			cp = psz[i];
		if(psz[i] == L'.'
//				|| std::binary_search(listAlphas.begin(), listAlphas.end(), cp)
				|| CEditView::IsIdentifierContinueChar(cp)) {
			if(cp > 0xFFFF)
				++i;
		}
		else
			return i;
	}
	return cch;
}

/**
 *	Zq̒Ԃ
 *	@param psz	ׂ镶
 *	@param cch	<var>psz</var> ̒
 *	@return		Zq̒BȂ0
 */
inline unsigned long CEditView::IsOperator(const wchar_t* psz, unsigned long cch) const {
	AssertValid();
	assert(psz != 0);

	if(cch == 0)
		return 0;
	OperatorMap::const_iterator	itInis = m_FoundationInfo.mapOperators.find(psz[0]);
	if(itInis == m_FoundationInfo.mapOperators.end())
		return 0;
	std::list<std::wstring>::const_iterator	it = itInis->second.begin();
	for(; it != itInis->second.end(); ++it) {
		if(cch < it->length())
			continue;
		if(0 == CompareString(psz, it->c_str(), it->length()))
			return it->length();
	}
	return 0;
}

/**
 *	㏑[hǂԂ
 *	@see	CEditView::SetOvertypeMode
 */
inline bool CEditView::IsOvertypeMode() const {
	AssertValid();
	return m_ModeState.bOvertype;
}

/**
 *	񂪒PsRg1JnŎn܂Ă邩Ԃ
 *	@param psz	ׂ镶
 *	@param cch	<var>psz</var> ̒
 */
inline bool CEditView::IsSComment1(const wchar_t* psz, unsigned long cch) const {
	assert(psz != 0);

	if(m_FoundationInfo.strCommentLineHeader1.empty())
		return false;
	if(cch < m_FoundationInfo.strCommentLineHeader1.length())
		return false;
	return 0 == CompareString(psz,
		m_FoundationInfo.strCommentLineHeader1.c_str(),
		m_FoundationInfo.strCommentLineHeader1.length());
}

/**
 *	񂪒PsRg2JnŎn܂Ă邩Ԃ
 *	@param psz	ׂ镶
 *	@param cch	<var>psz</var> ̒
 */
inline bool CEditView::IsSComment2(const wchar_t* psz, unsigned long cch) const {
	assert(psz != 0);

	if(m_FoundationInfo.strCommentLineHeader2.empty())
		return false;
	if(cch < m_FoundationInfo.strCommentLineHeader2.length())
		return false;
	return 0 == CompareString(psz,
		m_FoundationInfo.strCommentLineHeader2.c_str(),
		m_FoundationInfo.strCommentLineHeader2.length());
}

/**
 *	VONH[gň͂܂ꂽ̒Ԃ (pɓ)B
 *	GXP[vɂ̓obNXbVgpB
 *	Ȃ 0 ԂB
 *	ĂȂ <var>cch</var> 𓯂lԂ
 */
inline unsigned long CEditView::IsSingleQuotation(const wchar_t* psz, unsigned long cch) {
	assert(psz != 0);

	if(*psz != L'\'')
		return 0;
	for(unsigned long i = 1; i < cch; ++i) {
		if(*(psz + i) == L'\\')	// ͖̕
			++i; 
		else if(*(psz + i) == L'\'')
			return i + 1;
	}
	return cch;
}

} // namespace Ascension

#endif /* _EDIT_VIEW_H_ */

/* [EOF] */